#pragma warning(disable :4786)

#include <iostream>		// cout, <<
#include <fstream>		// ifstream, getline, open, close
#include <vector>		// vector, push_back, operator[], operator=
#include <string>		// string, operator+=, operator[], npos,
						// find_first_0f
using namespace std;

//----< look for '#' in line >-------------------------------------
//
// precondition:
//   assumes formatted IO was used (getline, not get)
//   so there is no leading white space

bool foundHash(const string &line) { return (line[0] == '#'); }

//----< look for '\"' or '<' in line, return postiion pos >--------

bool foundDelim(const string &line, int &pos) {

  pos = line.find_first_of('<');
  if(pos == string::npos)
    pos = line.find_first_of('\"');
  return (pos!= string::npos);
}

//----< return name from #include line, including delims >---------
//
// precondition:
//   at invocation pos points to the delimiter
//   just before name, e.g., '<' or '\"'

string extractName(const string &s, int pos) {
  string temp;
  while(pos < s.size()) {
    temp += s[pos++];
    if(s[pos] == '>' || s[pos] == '\"') {
      temp += s[pos];
      break;
    }
  }
  return temp;
}

//----< shorthand aliases >----------------------------------------

typedef vector< vector< string > > Grid;
typedef vector< string > Row;

//----< store names from include statements >---------------------
//
// post condition:
//   store file name as first item in row, includes follow

void getIncludesFromFile(char *filename, Row &row) {

  static ifstream in;			//create once on first entry
  row.push_back(filename);		//save file name as first item
  in.open(filename);
  string line;					// storage for lines read from file
  while(in.good()) {
    getline(in,line);
    int pos;
    if(foundHash(line) && foundDelim(line, pos)) {
      string name = extractName(line,pos);
      row.push_back(name);		//save include file names
    }
  }
  in.close();
  in.clear();					//necessary to reuse stream
}

//----< display dependency data >--------------------------------
void display(const Grid &grid) {

  int i;
  for(i=0; i<grid.size(); i++) {
    cout << " " << grid[i][0] << "\n";
    int j;
    for(j=1; j<grid[i].size(); j++)
      cout << " " << grid[i][j];
      cout << "\n\n";
  }
}

//----< insertion operator for grids >---------------------------
ostream& operator<<(ostream &out, const Grid &grid) {
  display(grid);
  return out;
}

//----< swap rows of grid >--------------------------------------
void swap(Grid &grid, int row1, int row2) {

  if(0<= row1 && row1 < grid.size())
    if(0<= row2 && row2 < grid.size()) {
      Row temp = grid[row1];
      grid[row1] = grid[row2];
      grid[row2] = temp;
    }
}
